package de.monarchy.guideme.data;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import de.monarchy.guideme.util.GeoCoordinate;

/**
 * PositionListener is an internal class implementing interfaces for sensor and
 * location events. It is bound to a PL service and feeds it with information
 * about changes in location and orientation. Its only purpose is to work as an
 * adapter class.
 * 
 * @author dstuerze
 * 
 */
public class PositionListener implements SensorEventListener, LocationListener {

  private PersonalLocation pl;

  /**
   * Creates a new listener object for this particular PersonalLocation.
   * 
   * @param p
   *          service the listener is applied to
   */
  public PositionListener(PersonalLocation p) {
    if (p == null)
      throw new NullPointerException(
          "PositionListener: paramater must not be null");
    pl = p;
  }

  @Override
  public void onAccuracyChanged(Sensor arg0, int arg1) {
    // in this case, we got absolutely nothing to do...
  }

  @Override
  public void onSensorChanged(SensorEvent event) {
    pl.handler
        .obtainMessage(PersonalLocation.ORIENTATION_CHANGED, event.values)
        .sendToTarget();
  }

  @Override
  public void onLocationChanged(Location arg0) {
    GeoCoordinate arg = new GeoCoordinate(arg0.getLatitude(), arg0
        .getLongitude());
    pl.handler.obtainMessage(PersonalLocation.POSITION_CHANGED, arg)
        .sendToTarget();
  }

  @Override
  public void onProviderDisabled(String arg0) {
    LocationManager lm = (LocationManager) pl.context
        .getSystemService(Context.LOCATION_SERVICE);
    lm.removeUpdates(this);
    pl.registerPositionListener();
  }

  @Override
  public void onProviderEnabled(String arg0) {
    // in this case, we got absolutely nothing to do...
  }

  @Override
  public void onStatusChanged(String arg0, int arg1, Bundle arg2) {
    if (arg1 == LocationProvider.OUT_OF_SERVICE) {
      LocationManager lm = (LocationManager) pl.context
          .getSystemService(Context.LOCATION_SERVICE);
      lm.removeUpdates(this);
      pl.registerPositionListener();
    }
  }

}
