package de.monarchy.guideme.map.nutiteq;

import static de.monarchy.guideme.view.CombinationView.CAM_THRESHOLD;
import static de.monarchy.guideme.view.CombinationView.MAP_THRESHOLD;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.microedition.lcdui.Image;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;
import android.location.Location;
import android.location.LocationManager;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.RelativeLayout;
import android.widget.ZoomControls;

import com.nutiteq.BasicMapComponent;
import com.nutiteq.android.MapView;
import com.nutiteq.components.Line;
import com.nutiteq.components.Place;
import com.nutiteq.components.WgsPoint;
import com.nutiteq.controls.AndroidKeysHandler;
import com.nutiteq.listeners.MapListener;
import com.nutiteq.maps.CloudMade;
import com.nutiteq.ui.Pannable;
import com.nutiteq.ui.PanningStrategy;

import de.monarchy.guideme.R;
import de.monarchy.guideme.map.MapWrapper;
import de.monarchy.guideme.util.GeoCoordinate;
import de.monarchy.guideme.util.POI;
import de.monarchy.guideme.util.RoutePoint;
import de.monarchy.guideme.view.CombinationView;

/**
 * A map wrapper for the Nutiteq api.
 * 
 * @author dstuerze
 * 
 */
public class NutiteqWrapper extends MapWrapper {

  // map model
  private BasicMapComponent mapModel;
  // map view
  private ExtendedMapView mapView;
  private RelativeLayout mapLayout;
  private ZoomControls mapZooms;
  private Image posIcon;
  private Image poiIcon;
  // data
  private GeoCoordinate position;
  private Place pos;
  private int defaultHeight;
  private int panHeight;
  private Line route;
  private Set<POI> pois = new HashSet<POI>();

  // paint data for viewing indicator
  private static Paint paint;

  // this is a singleton instance
  private static NutiteqWrapper instance;

  static {
    // create a grey color
    paint = new Paint();
    paint.setColor(0x80A0B0D0);
  }

  /**
   * Initializes an instance.
   * 
   * @param context
   */
  private NutiteqWrapper(Context context) {
    super();

    try {
      // position indicator
      InputStream in = context.getResources().openRawResource(
          R.drawable.red_dot);
      posIcon = Image.createImage(in);
      in.close();
      // poi indicator
      in = context.getResources().openRawResource(R.drawable.blue_dot);
      poiIcon = Image.createImage(in);
      in.close();
    } catch (IOException e) {
      Log.i("GuideMe", "NutiteqWrapper: no place icons found");
    }

    // calculate height and width
    WindowManager m = (WindowManager) context
        .getSystemService(Context.WINDOW_SERVICE);
    int defaultWidth = m.getDefaultDisplay().getWidth();
    defaultHeight = m.getDefaultDisplay().getHeight();
    panHeight = 0;

    // find an initial position
    LocationManager lm = (LocationManager) context
        .getSystemService(Context.LOCATION_SERVICE);
    List<String> l = lm.getAllProviders();
    long time = 0;
    Location lo = null;
    if (lm.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
      // try to obtain gps data
      lo = lm.getLastKnownLocation(LocationManager.GPS_PROVIDER);
    } else {
      // or use the last known location
      for (String p : l) {
        Location loc = lm.getLastKnownLocation(p);
        if (loc != null)
          if (time <= loc.getTime()) {
            time = loc.getTime();
            lo = loc;
          }
      }
    }
    double lon = lo != null ? lo.getLongitude() : 8.028985;
    double lat = lo != null ? lo.getLatitude() : 52.27126;
    position = new GeoCoordinate(lat, lon);
    pos = new Place(0, "Position", posIcon, lon, lat);

    // create the model
    WgsPoint w = new WgsPoint(lon, lat);
    mapModel = new BasicMapComponent(
        "c0e190d8267e36708f955d7ab048990d4b6ad8cf73dcc2.49279895", "monarchy",
        "GuideMe", defaultWidth, defaultHeight, w, 14);
    Log.d("GuideMe", "NutiteqWrapper map width=" + defaultWidth + " height="
        + defaultHeight);
    mapModel.setMap(new CloudMade("d23f55008e5949869a7d0b15e45f0b20", 256, 1));
    mapModel.setControlKeysHandler(new AndroidKeysHandler());
    // forbid scrolling with wheel
    mapModel.setPanningStrategy(new PanningStrategy() {
      @Override
      public boolean isPanning() {
        return false;
      }

      @Override
      public void keyRepeated(int arg0) {
      }

      @Override
      public void quit() {
      }

      @Override
      public void setMapComponent(Pannable arg0) {
      }

      @Override
      public void start() {
      }

      @Override
      public void startPanning(int arg0, int arg1, boolean arg2) {
      }

      @Override
      public void stopPanning() {
      }
    });
    mapModel.setTouchClickTolerance(BasicMapComponent.FINGER_CLICK_TOLERANCE);
    // register touch delegate
    mapModel.setMapListener(new MapListener() {
      @Override
      public void mapClicked(WgsPoint arg0) {
        if (touchDelegate != null) {
          GeoCoordinate g = new GeoCoordinate(arg0.getLat(), arg0.getLon());
          touchDelegate.onTouch(g);
        }
      }

      @Override
      public void mapMoved() {
      }

      @Override
      public void needRepaint(boolean arg0) {
      }
    });
    // show position
    mapModel.addPlace(pos);
    try {
      mapModel.startMapping();
    } catch (Exception e) {
      Log.e("GuideMe", "NutiteqWrapper mapping not started!");
      e.printStackTrace();
    }

    // build map view
    mapLayout = new RelativeLayout(context);
    mapLayout.setClipChildren(false);
    mapLayout.setClipToPadding(false);
    mapView = new ExtendedMapView(context, mapModel);

    // add zoom controls
    mapZooms = new ZoomControls(context);
    mapZooms.setOnZoomInClickListener(new View.OnClickListener() {
      public void onClick(final View v) {
        mapModel.zoomIn();
      }
    });
    mapZooms.setOnZoomOutClickListener(new View.OnClickListener() {
      public void onClick(final View v) {
        mapModel.zoomOut();
      }
    });
    mapLayout.addView(mapView, defaultWidth, defaultHeight);
    RelativeLayout.LayoutParams zoomParams = new RelativeLayout.LayoutParams(
        ViewGroup.LayoutParams.WRAP_CONTENT,
        ViewGroup.LayoutParams.WRAP_CONTENT);
    zoomParams.addRule(RelativeLayout.ALIGN_PARENT_TOP);
    zoomParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
    mapLayout.addView(mapZooms, zoomParams);

  }

  @Override
  public void onOrientationChanged(float[] arg) {
    if (!isDrawing)
      return;
    // get viewing direction
    mapView.angle = (int) arg[0];
    // move map according to the height of the view
    float roll;
    if (arg[2] < CombinationView.CAM_THRESHOLD || isFullScreen)
      roll = 1;
    else
      roll = ((MAP_THRESHOLD - CAM_THRESHOLD) - (arg[2] - CAM_THRESHOLD))
          / (MAP_THRESHOLD - CAM_THRESHOLD);
    int nextPan = (defaultHeight - (int) (defaultHeight * roll)) / 2;
    mapModel.panMap(0, nextPan - panHeight);
    panHeight = nextPan;
    mapView.invalidate();
  }

  @Override
  public void onPositionChanged(GeoCoordinate arg, POI[] arg1) {
    // put all pois on the map
    for (int i = 0; i < arg1.length; i++) {
      if (!pois.contains(arg1[i])) {
        pois.add(arg1[i]);
        Place p = new Place(pois.size(), arg1[i].getDescription(), poiIcon,
            arg1[i].getLongitude(), arg1[i].getLatitude());
        mapModel.addPlace(p);
      }
    }
    // set the new position
    position = arg;
    mapModel.removePlace(pos);
    pos = new Place(0, "Position", posIcon, position.getLongitude(), position
        .getLatitude());
    mapModel.addPlace(pos);
    // center the map
    mapModel.moveMap(arg.getLongitude(), arg.getLatitude());
    mapModel
        .setMiddlePoint(new WgsPoint(arg.getLongitude(), arg.getLatitude()));
    mapModel.panMap(0, panHeight);
    mapView.invalidate();
  }

  /**
   * View extension of Nutiteq's standard map view. Its onDraw method is able to
   * show the user where he is looking at.
   * 
   * @author dstuerze
   * 
   */
  private class ExtendedMapView extends MapView {

    // view direction
    int angle = 0;

    /**
     * Builds the view for the given context, obtaining data from the given
     * model.
     * 
     * @param context
     *          application context
     * @param component
     *          model providing data
     */
    public ExtendedMapView(Context context, BasicMapComponent component) {
      super(context, component);
    }

    protected void onDraw(Canvas c) {
      // height
      Rect r = c.getClipBounds();
      int h = r.bottom - r.top;
      // width
      int w = r.right - r.left;
      try {
        // draw map
        super.onDraw(c);
      } catch (Exception e) {
        Log.w("GuideMe", "NutiteqWrapper map drawing failed");
      }
      if (showViewDirection) {
        // overlay with arc pointing into viewing direction
        RectF rf = new RectF((w - h) / 2, 0, w - (w - h) / 2, h);
        c.drawArc(rf, angle - 20, 40, true, paint);
      }
    }

  }

  public View getView() {
    return mapLayout;
  }

  public void removeRoute() {
    mapModel.removeLine(route);
  }

  @Override
  public void setRoute(RoutePoint[] routePoints) {
    if (routePoints == null || routePoints.length == 0)
      return;
    WgsPoint[] points = new WgsPoint[routePoints.length];
    for (int i = 0; i < routePoints.length; i++) {
      points[i] = new WgsPoint(routePoints[i].getLongitude(), routePoints[i]
          .getLatitude());
    }
    mapModel.removeLine(route);
    route = new Line(points);
    mapModel.addLine(route);
    mapView.invalidate();
  }

  public void setScrollable(boolean b) {
    super.setScrollable(b);
    if (b)
      mapModel.setTouchClickTolerance(BasicMapComponent.FINGER_CLICK_TOLERANCE);
    else
      mapModel.setTouchClickTolerance(10000);
  }

  public void centerAt(GeoCoordinate g) {
    WgsPoint arg0 = new WgsPoint(g.getLongitude(), g.getLatitude());
    mapModel.setMiddlePoint(arg0);
  }

  @Override
  public GeoCoordinate getCenter() {
    WgsPoint c = mapModel.getCenterPoint();
    return new GeoCoordinate(c.getLat(), c.getLon());
  }

  public void positionZoomControls(int x, int y) {
    RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
        ViewGroup.LayoutParams.WRAP_CONTENT,
        ViewGroup.LayoutParams.WRAP_CONTENT);
    params.addRule(x);
    params.addRule(y);
    mapZooms.setLayoutParams(params);
    mapLayout.invalidate();
  }

  /**
   * Returns an instance of the NutiteqWrapper. NutiTeqWrapper is a singleton,
   * so it only creates one instance.
   * 
   * @param c
   *          application context
   * @return the one and only instance
   */
  public static NutiteqWrapper getInstance(Context c) {
    if (instance == null)
      instance = new NutiteqWrapper(c);
    return instance;
  }

  protected void finalize() {
    mapModel.stopMapping();
  }

}
