package de.monarchy.guideme.pois;

import de.monarchy.guideme.util.GeoCoordinate;
import de.monarchy.guideme.util.POI;

/**
 * This interface describes a simple provider for POIs.
 * Classes implementing this interface have to follow one important convention:
 * they have to be instanciated with a default constructor. If no default
 * constructor exists, reflective creation will not be performed, so no 
 * data will be received.
 * 
 * @author dstuerze
 *
 */
public interface POIProvider {
  /**
   * This method is called by anyone who wants to receive points of interest
   * in a certain radius around a specified location.
   * @param where central point of search
   * @param radius distance in metres around this spot
   * @param what specification of what to search (optional)
   * @return array of POIs
   */
  POI[] receivePOIs(GeoCoordinate where, int radius, String what);
}
