package de.monarchy.guideme.pois.wikipedia;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import de.monarchy.guideme.util.GeoCoordinate;
import de.monarchy.guideme.util.POI;

/**
 * This class is a content handler for sax xml parsing. It is specialized for
 * the wikipedia web service.
 * 
 * @author dstuerze
 * 
 */
public class WikiHandler implements ContentHandler {

  // all those articles found
  private Set<WikipediaArticle> articles = new HashSet<WikipediaArticle>();
  // attributes of the article parsing now
  private Map<String, String> attributes = new TreeMap<String, String>();
  private StringBuffer buffer;

  /**
   * Returns the entries found.
   * 
   * @return all articles as array
   */
  public POI[] getEntries() {
    return articles.toArray(new POI[0]);
  }

  @Override
  public void characters(char[] ch, int start, int length) throws SAXException {
    buffer.append(ch, start, length);
  }

  @Override
  public void startElement(String uri, String localName, String qName,
      Attributes atts) throws SAXException {
    if (localName.equals("entry")) {
      // entry means: new article
      attributes = new TreeMap<String, String>();
    } else {
      // else parse content
      buffer = new StringBuffer();
    }

  }

  @Override
  public void endElement(String uri, String localName, String qName)
      throws SAXException {
    if (localName.equals("entry")) {
      // entry means: that's a whole article
      // find the attributes for latitude and longitude
      double lat = Double.parseDouble(attributes.get("lat"));
      double lon = Double.parseDouble(attributes.get("lng"));
      GeoCoordinate gc = new GeoCoordinate(lat, lon);
      // find the article's title, summary and url
      String name = attributes.get("title");
      String summary = attributes.get("summary");
      String wikiUrl = attributes.get("wikipediaUrl");
      URL url;
      try {
        url = new URL(wikiUrl);
      } catch (MalformedURLException e) {
        return;
      }
      // build the article and add it to the list
      WikipediaArticle put = new WikipediaArticle(gc, name, url, summary);
      articles.add(put);
    } else {
      // here we only found an attribute
      attributes.put(localName, buffer.toString());
    }
  }

  // What follows are empty methods!
  @Override
  public void endDocument() throws SAXException {
  }

  @Override
  public void startDocument() throws SAXException {
  }

  @Override
  public void endPrefixMapping(String prefix) throws SAXException {
  }

  @Override
  public void ignorableWhitespace(char[] ch, int start, int length)
      throws SAXException {
  }

  @Override
  public void processingInstruction(String target, String data)
      throws SAXException {
  }

  @Override
  public void setDocumentLocator(Locator locator) {
  }

  @Override
  public void skippedEntity(String name) throws SAXException {
  }

  @Override
  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

}
