package de.monarchy.guideme.routing;

import de.monarchy.guideme.util.PointOfInterest;

/**
 * The interface GeoCodingListener provides a method for callback functions.
 * When a GeoCodingComponent receives an asynchronous call for places, it
 * expects a GeoCodingListener to call back. After the places are found, it
 * calls the onPlacesFound method to inform the callback component.
 * 
 * @author dstuerze
 * 
 */
public interface GeoCodingListener {
  /**
   * Delivers an array of places which fit a certain criteria around a specified
   * position.
   * 
   * @param places
   *          array of possible matches for the search
   * @param search
   *          the search string
   */
  public void onPlacesFound(String search, PointOfInterest... places);

  /**
   * This method is called if an error occured while finding geocoded places.
   * 
   * @param code
   *          a constant value from GeoCodingComponent indicating the type of
   *          error
   */
  public void onError(int code);

}
