package de.monarchy.guideme.routing;

import de.monarchy.guideme.util.RoutePoint;

/**
 * The interface RoutingListener provides a method for callback functions. When
 * a RoutingComponent receives an asynchronous call for a route, it expects a
 * RouteListener to call back. After the route is calculated, it calls the
 * onRouteCalculated method to inform the callback component.
 * 
 * @author dstuerze
 * 
 */
public interface RoutingListener {

  /**
   * Delivers a route to the Listener.
   * 
   * @param points
   *          RoutePoint[] with turning points
   */
  public void onRouteCalculated(RoutePoint[] points);

  /**
   * This method is called if an error occured while calculating the route.
   * 
   * @param code
   *          a constant value from RoutingComponent indicating the type of
   *          error
   */
  public void onError(int code);

}
