package de.monarchy.guideme.util;

import java.io.Serializable;

import android.content.Context;
import android.view.View;

/**
 * POI declares a general named location. Every location that needs to be
 * described, should implement this interface.
 * 
 * @author dstuerze
 * 
 */
public interface POI extends Serializable {

  /**
   * Returns a description of the location in a String.
   * 
   * @return short description
   */
  String getDescription();

  /**
   * Returns the latitude of this location. Values are supposed to range from
   * -90. to +90. with positive values denoting locations on the northern
   * hemisphere.
   * 
   * @return latitude in degrees from -90 to 90
   */
  double getLatitude();

  /**
   * Returns the longitude of this location.. Values are supposed to range from
   * -180. to +180. with positve values denoting a position east of Greenwich
   * and negative ones positions in the west.
   * 
   * @return longitude in degrees from -180 to 180
   */
  double getLongitude();

  /**
   * Returns the position of this location in form of a GeoCoordinate. It
   * contains information about latitude and longitude in an encoding matching
   * this one's.
   * 
   * @return GeoCoordinate with the location
   */
  GeoCoordinate getLocation();

  /**
   * Returns a graphic version of this point of interest. If the POI does not
   * have a particular view, this method will return null. Calling components
   * therefore need to check that.
   * 
   * @param c
   *          application context
   * @return view of this POI or null, if none exists
   */
  View getView(Context c);
}