package de.monarchy.guideme.util;

/**
 * RoutePoint is a special type of POI. It represents routing information
 * about turning points and similar. In addition to position and 
 * description, it also holds information about the distance to the next 
 * point.
 * 
 * @author dstuerze
 *
 */
public class RoutePoint extends PointOfInterest {

  /** */
  private static final long serialVersionUID = 3391237934695587639L;
  
  /** distance to next route point */
  private double distance;
  
  /**
   * Creates a new route point with the given information. As every POI,
   * it needs a location and description. Additionally, it holds 
   * information about the distance to the next POI on the route. The
   * description can be used for instructions.
   * 
   * @param lat latitude of the location
   * @param lon longitude of the location
   * @param descr description
   * @param dist distance to next point, >= 0.
   * @throws IllegalArgumentException if dist < 0.
   */
  public RoutePoint(double lat, double lon, String descr, double dist) {
    super(lat, lon, descr);
    if (dist < 0.)
      throw new IllegalArgumentException("RoutePoint: dist < 0.");
    distance = dist;
  }
  
  /**
   * Creates a new route point with the given information. As every POI,
   * it needs a location and description. Additionally, it holds 
   * information about the distance to the next POI on the route. The
   * description can be used for instructions.
   * 
   * @param gc location
   * @param descr description
   * @param dist distance to next point, >= 0.
   * @throws IllegalArgumentException if dist < 0.
   */
  public RoutePoint(GeoCoordinate gc, String descr, double dist) {
    this(gc.getLatitude(), gc.getLongitude(), descr, dist);
  }

  /**
   * Returns the distance to the next route point. 
   * @return distance to next route point
   */
  public double getDistance() {
    return distance;
  }
}
